import 'dart:convert';
import 'dart:ffi';
import 'package:ffi/ffi.dart';
import 'dart:typed_data';
import 'array_utils.dart';

/// Thin wrapper arount a [Pointer<T>] and its associated typed list [L]
class Array<T extends NativeType, L extends List<int>> {
  /// Pointer to array
  final Pointer<T> ptr;

  /// View into array
  final L view;

  const Array.fromPtr(this.ptr, this.view);

  /// Allocates Array of [length].
  /// [getView] returns the typed list of [Pointer<T>]
  factory Array(int length, L Function(Pointer<T>, int) getView) {
    final ptr = allocate<T>(count: length);
    final view = getView(ptr, length);
    return Array.fromPtr(ptr, view);
  }
  factory Array.fromList(L list, L Function(Pointer<T>, int) getView) {
    final arr = Array(list.length, getView);
    arr.view.setAll(0, list);
    return arr;
  }
}

class Uint8Array extends Array<Uint8, Uint8List> {
  const Uint8Array.fromPtr(Pointer<Uint8> ptr, Uint8List view)
      : super.fromPtr(ptr, view);
  factory Uint8Array(int length) => Array(length, getUint8View);
  factory Uint8Array.fromStr(String str) {
    final arr = Uint8Array(str.length);
    final utf8enc = utf8.encode(str);
    arr.view.setAll(0, utf8enc);
    return arr;
  }
  factory Uint8Array.fromUint8(Uint8List list) =>
      Array.fromList(list, getUint8View);
}
